package com.macro.mall.tiny.modules.business.controller;

import com.google.common.collect.Lists;
import com.macro.mall.tiny.common.api.CommonResult;
import com.macro.mall.tiny.common.util.MyFileUtils;
import com.macro.mall.tiny.config.UploadConfig;
import com.macro.mall.tiny.modules.business.dto.FileDownloadParam;
import com.macro.mall.tiny.modules.business.dto.FileUploadParam;
import com.macro.mall.tiny.modules.business.model.BDirectory;
import com.macro.mall.tiny.modules.business.model.BFile;
import com.macro.mall.tiny.modules.business.model.BFileDetail;
import com.macro.mall.tiny.modules.business.service.FileService;
import com.macro.mall.tiny.security.util.ShiroAuthUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.compress.archivers.zip.ParallelScatterZipCreator;
import org.apache.commons.compress.archivers.zip.ZipArchiveEntry;
import org.apache.commons.compress.archivers.zip.ZipArchiveOutputStream;
import org.apache.commons.compress.parallel.InputStreamSupplier;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.ClassPathResource;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URLEncoder;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.zip.ZipEntry;

/**
 * 大文件上传
 */
@Slf4j
@RestController
@RequestMapping("/file")
@CrossOrigin
@Api(tags = "FileController", description = "提供文件上传相关API")
public class FileController {
    @Autowired
    private FileService fileService;

    /**
     * @param fileUploadParam
     * @return void
     * @describe 文件上传
     * @author gjs
     * @date 2021/3/25
     * @since J7.23.0
     */
    @ApiOperation("文件上传接口")
    @PostMapping("/upload")
    public CommonResult<String> upload(FileUploadParam fileUploadParam) throws IOException {
        String name = fileUploadParam.getName();
        String lineName = fileUploadParam.getLineName();
        String provinceId = fileUploadParam.getProvince();
        String cityId = fileUploadParam.getCity();
        String voltage = fileUploadParam.getVoltage();
        String lineStatus = fileUploadParam.getLineStatus();
        Long serviceBegintimeTs = fileUploadParam.getServiceBegintimeTs();
        Integer beginTower = fileUploadParam.getBeginTower();
        Integer endTower = fileUploadParam.getEndTower();
        Long size = fileUploadParam.getSize();
        Integer chunks = fileUploadParam.getChunks();
        Integer chunk = fileUploadParam.getChunk();
        MultipartFile file = fileUploadParam.getFile();
        String charset = fileUploadParam.getCharset();

        if (chunks != null && chunks != 0) {
            fileService.uploadWithBlock(name, lineName, provinceId, cityId, voltage, lineStatus, serviceBegintimeTs, beginTower, endTower, size, chunks, chunk, file, charset);
        } else {
            fileService.upload(file);
        }
        return CommonResult.success("上传成功，后台正在处理...");
    }

    /**
     * 下载模板
     */
    @ApiOperation("下载杆塔信息录入模板")
    @GetMapping("/downloadTemplate")
    public void downloadTemplate(HttpServletResponse response) throws Exception {
        ClassPathResource classPathResource = new ClassPathResource("excel-template/杆塔信息录入.xlsx");
        InputStream inputStream = classPathResource.getInputStream();
        Workbook workbook = new XSSFWorkbook(inputStream);
        response.setContentType("application/vnd.ms-excel");
        response.setHeader("content-Disposition", "attachment;filename=" + URLEncoder.encode("杆塔信息录入.xlsx", "utf-8"));
        response.setHeader("Access-Control-Expose-Headers", "content-Disposition");
        OutputStream outputStream = response.getOutputStream();
        workbook.write(outputStream);
        outputStream.flush();
        outputStream.close();
    }

    /**
     * @param
     * @return com.macro.mall.tiny.common.api.CommonResult<com.macro.mall.tiny.modules.business.model.BDirectory>
     * @describe 获取全部文件夹
     * @author gjs
     * @date 2021/4/2
     * @since J7.23.0
     */
    @ApiOperation("获取全部文件夹接口")
    @GetMapping("/getAllDirectory")
    public CommonResult<BDirectory> getDirectory() {
        final BDirectory directory = fileService.getRootDirectory();
        return CommonResult.success(directory);
    }

    /**
     * @param params
     * @return com.macro.mall.tiny.common.api.CommonResult<java.util.List < com.macro.mall.tiny.modules.business.model.BFileDetail>>
     * @describe 获取文件夹下全被文件详细接口
     * @author gjs
     * @date 2021/4/5
     * @since J7.23.0
     */
    @ApiOperation("获取文件夹下全部文件详细接口")
    @PostMapping("/getFileDetail")
    public CommonResult<List<BFileDetail>> getFileDetail(@RequestBody Map<String,String> params) {
        List<BFileDetail> fileDetailList = fileService.getFileDetailList(params.get("path"));
        return CommonResult.success(fileDetailList);
    }

    @ApiOperation("获取文件夹下全部文件/文件夹")
    @PostMapping("/getFile")
    public CommonResult<List<BFile>> getFile(@RequestBody Map<String,String> params) {
        List<BFile> fileDetailList = fileService.getFileList(params.get("path"));
        return CommonResult.success(fileDetailList);
    }

    @ApiOperation("文件下载接口")
    @PostMapping("/download")
    public void downLoad(HttpServletResponse response, @RequestBody Map<String,String> params) throws Exception {
        boolean access = ShiroAuthUtil.checkAccess(ShiroAuthUtil.ACCESS);
        File file = new File(params.get("path"));
        if (file.getName().contains(String.valueOf(UploadConfig.AUTH_STR)) && !access) return;
        if (file.exists()) { //判断文件父目录是否存在
            String fileName = file.getName();
            response.setContentType("application/form-data");
            response.setCharacterEncoding("UTF-8");
            response.setHeader("Content-Disposition", "attachment;fileName=" + URLEncoder.encode(fileName, "UTF-8"));
            response.setHeader("Access-Control-Expose-Headers", "content-Disposition");
            try (FileInputStream fileInputStream = new FileInputStream(file)) {
                try (BufferedInputStream bufferedInputStream = new BufferedInputStream(fileInputStream)) {
                    try (OutputStream outputStream = response.getOutputStream()) {
                        byte[] buffer = new byte[1024];
                        int read = bufferedInputStream.read(buffer);
                        while (read != -1) {
                            outputStream.write(buffer);
                            read = bufferedInputStream.read(buffer);
                        }

                    }
                }
            }
        }
    }

    @ApiOperation("多文件压缩下载接口")
    @PostMapping("/multiFileZipDownload")
    public void multiFileZipDownload(HttpServletResponse response, @Validated @RequestBody FileDownloadParam fileDownloadParam) throws Exception {
        boolean access = ShiroAuthUtil.checkAccess(ShiroAuthUtil.ACCESS);
        OutputStream outputStream = response.getOutputStream();
        ParallelScatterZipCreator scatterZipCreator = new ParallelScatterZipCreator();
        ZipArchiveOutputStream zipArchiveOutputStream = new ZipArchiveOutputStream(outputStream);
        response.setContentType("multipart/form-data");
        response.setHeader("Content-Disposition", "attachment;fileName=" + URLEncoder.encode(fileDownloadParam.getName(), "UTF-8"));
        response.setHeader("Access-Control-Expose-Headers", "content-Disposition");
        try {
            List<String> fileList = fileDownloadParam.getFileList();
            ArrayList<String> allSubFile = Lists.newArrayList();
            fileList.forEach(filePath -> {
                File file = new File(filePath);
                if (file.exists() && file.isDirectory()) {
                    MyFileUtils.addAllSubFileList(allSubFile, file, access);
                }
            });
            fileList.addAll(allSubFile);

            fileList.forEach(filePath -> {
                Path path = Paths.get(filePath);
                if (Files.isDirectory(path)) {
                    return;
                }
                Path relativePath = path.subpath(UploadConfig.getPowerPathCount() + 1, path.getNameCount());
                ZipArchiveEntry archiveEntry = new ZipArchiveEntry(relativePath.toString());
                archiveEntry.setMethod(ZipEntry.DEFLATED);
                InputStreamSupplier supplier = () -> {
                    try {
                        return new FileInputStream(path.toFile());
                    } catch (FileNotFoundException e) {
                        e.printStackTrace();
                        return null;
                    }
                };
                scatterZipCreator.addArchiveEntry(archiveEntry, supplier);
            });
            scatterZipCreator.writeTo(zipArchiveOutputStream);
        } finally {
            zipArchiveOutputStream.close();
            outputStream.close();
        }
    }

    @ApiOperation("文件删除接口")
    @PostMapping("/delete")
    public CommonResult<String> delete(@RequestBody Map<String,String> params) throws IOException {
        boolean deleteFile = fileService.deleteFile(params.get("path"));
        if (deleteFile) {
            return CommonResult.success("删除成功");
        } else {
            return CommonResult.failed("删除失败");
        }
    }

}
