package com.macro.mall.tiny.modules.business.controller;


import com.macro.mall.tiny.common.api.CommonResult;
import com.macro.mall.tiny.modules.business.dto.CommonIdParam;
import com.macro.mall.tiny.modules.business.dto.SensorQueryParam;
import com.macro.mall.tiny.modules.business.model.KrBSensor;
import com.macro.mall.tiny.modules.business.service.KrBSensorService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author macro
 * @since 2021-10-20
 */
@RestController
@Api(tags = "KrBSensorController", description = "提供传感器相关API")
@RequestMapping("/device")
public class KrBSensorController {

    @Autowired
    private KrBSensorService sensorService;

    @ApiOperation("根据线路名称或传感器mac获取传感器列表")
    @PostMapping("/get")
    public CommonResult<List<KrBSensor>> get(@RequestBody SensorQueryParam sensorQueryParam) {
        String lineName = sensorQueryParam.getLineName();
        String deviceMac = sensorQueryParam.getDeviceMac();
        List<KrBSensor> sensorList = sensorService.get(lineName, deviceMac);
        return CommonResult.success(sensorList);
    }

    @ApiOperation("新增传感器")
    @PostMapping("/add")
    public CommonResult add(@RequestBody KrBSensor sensor) {
        boolean success = sensorService.save(sensor);
        if (success) {
            return CommonResult.success(null);
        }
        return CommonResult.failed();
    }

    @ApiOperation("修改传感器")
    @PostMapping("/update")
    public CommonResult update(@RequestBody KrBSensor sensor) {
        boolean success = sensorService.updateById(sensor);
        if (success) {
            return CommonResult.success(null);
        }
        return CommonResult.failed();
    }

    @ApiOperation("删除传感器")
    @PostMapping("/delete")
    public CommonResult delete(@RequestBody CommonIdParam idParam) {
        Long id = idParam.getId();
        boolean success = sensorService.removeById(id);
        if (success) {
            return CommonResult.success(null);
        }
        return CommonResult.failed();
    }
}

