package com.macro.mall.tiny.modules.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import com.macro.mall.tiny.modules.business.mapper.KrBLineMapper;
import com.macro.mall.tiny.modules.business.mapper.KrBSensorMapper;
import com.macro.mall.tiny.modules.business.mapper.KrBTowerMapper;
import com.macro.mall.tiny.modules.business.model.KrBLine;
import com.macro.mall.tiny.modules.business.model.KrBSensor;
import com.macro.mall.tiny.modules.business.model.KrBTower;
import com.macro.mall.tiny.modules.business.service.KrBSensorService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author macro
 * @since 2021-10-20
 */
@Service
public class KrBSensorServiceImpl extends ServiceImpl<KrBSensorMapper, KrBSensor> implements KrBSensorService {

    @Autowired
    private KrBSensorMapper sensorMap;

    @Autowired
    private KrBLineMapper lineMapper;

    @Autowired
    private KrBTowerMapper towerMapper;

    @Override
    public List<KrBSensor> get(String lineName, String deviceMac) {
        LambdaQueryWrapper<KrBSensor> sensorQueryWrapper;
        List<KrBSensor> sensorList;
        if (!StringUtils.isBlank(deviceMac)) {
            sensorQueryWrapper = new QueryWrapper<KrBSensor>().lambda().eq(KrBSensor::getDeviceMac, deviceMac);
        } else {
            LambdaQueryWrapper<KrBLine> lineQueryWrapper = new QueryWrapper<KrBLine>().lambda().eq(KrBLine::getLineName, lineName);
            KrBLine line = lineMapper.selectOne(lineQueryWrapper);
            if (line==null) return Lists.newArrayList();
            Long lineId = line.getId();
            LambdaQueryWrapper<KrBTower> towerQueryWrapper = new QueryWrapper<KrBTower>().lambda().eq(KrBTower::getLineId, lineId);
            List<KrBTower> towerList = towerMapper.selectList(towerQueryWrapper);
            if (towerList==null||towerList.isEmpty()) return Lists.newArrayList();
            List<Long> towerIdList = towerList.stream().map(KrBTower::getId).collect(Collectors.toList());
            sensorQueryWrapper = new QueryWrapper<KrBSensor>().lambda().in(KrBSensor::getTowerId, towerIdList);
        }
        sensorList = sensorMap.selectList(sensorQueryWrapper);
        return sensorList;
    }

}
